@@ -19,14 +19,16 @@ class SumCov(FunctionReduce):
         self._covariances = [c for c in covariances]
         FunctionReduce.__init__(self, self._covariances, 'sum')
 
-    def value_reduce(self, values):  # pylint: disable=R0201
-        r"""Sum covariance function evaluated at `(f_0, f_1, ...)`."""
+    def value_reduce(self, values):
+        r"""Sum covariance function evaluated at ``(f_0, f_1, ...)``."""
         return add.reduce(list(values.values()))
 
-    def gradient_reduce(self, _, gradients):  # pylint: disable=R0201
+    def gradient_reduce(self, _, gradients):
         r"""Sum of covariance function derivatives.
 
-        Returns:
+        Returns
+        -------
+        dict
             :math:`f_0' + f_1' + \dots`
         """
         grad = dict()
