@@ -1,7 +1,7 @@
 from __future__ import division
 
 from glimix_core.cov import EyeCov
-from optimix.testing import Assertion
+from optimix import Assertion
 
 
 def test_eyecov_optimix():
