@@ -4,7 +4,7 @@ from numpy import zeros
 from numpy.testing import assert_allclose
 
 from glimix_core.cov import FreeFormCov
-from optimix.testing import Assertion
+from optimix import Assertion
 
 
 def test_freeform_optimix():
