@@ -3,11 +3,14 @@ from __future__ import absolute_import, division, unicode_literals
 from copy import copy
 
 from liknorm import LikNormMachine
-from numpy import exp
-from numpy import ascontiguousarray
+from numpy import asarray, ascontiguousarray, dot, exp
+from numpy.linalg import norm, solve
 
 from glimix_core.ep import EPLinearKernel
-from .glmm import GLMM
+from numpy_sugar import epsilon
+from numpy_sugar.linalg import sum2diag
+
+from ._glmm import GLMM
 
 
 class GLMMExpFam(GLMM):
@@ -15,7 +18,7 @@ class GLMMExpFam(GLMM):
 
     It implements inference over the GLMM explained in Section BLA via
     the Expectation Propagation [Min01]_ algorithm.
-    It currently supports the `Bernoulli`, `Binomial`, and `Poisson`
+    It currently supports the `Bernoulli`, `Probit`, `Binomial`, and `Poisson`
     likelihoods. (For heterogeneous Normal likelihood, please refer to
     :class:`glimix_core.glmm.GLMMNormal` for a closed-form inference.)
 
@@ -24,7 +27,8 @@ class GLMMExpFam(GLMM):
     y : array_like
         Outcome variable.
     lik_name : str
-        Likelihood name. It supports `Bernoulli`, `Binomial`, and `Poisson`.
+        Likelihood name. It supports `Bernoulli`, `Probit`, `Binomial`, and
+        `Poisson`.
     X : array_like
         Covariates.
     QS : tuple
@@ -67,12 +71,19 @@ class GLMMExpFam(GLMM):
         After: -13.43
     """
 
-    def __init__(self, y, lik_name, X, QS):
+    def __init__(self,
+                 y,
+                 lik_name,
+                 X,
+                 QS=None,
+                 n_int=1000,
+                 rtol=epsilon.small * 1000,
+                 atol=epsilon.small):
         GLMM.__init__(self, y, lik_name, X, QS)
 
-        self._ep = EPLinearKernel(self._X.shape[0])
+        self._ep = EPLinearKernel(self._X.shape[0], rtol=rtol, atol=atol)
         self._ep.set_compute_moments(self.compute_moments)
-        self._machine = LikNormMachine(self._lik_name, 1000)
+        self._machine = LikNormMachine(self._lik_name, n_int)
         self.update_approx = True
 
         self.variables().get('beta').listen(self.set_update_approx)
@@ -117,6 +128,19 @@ class GLMMExpFam(GLMM):
     def covariance(self):
         return dict(QS=self._QS, scale=self.scale, delta=self.delta)
 
+    def fit(self, verbose=True, factr=1e5, pgtol=1e-7):
+        r"""Maximise the marginal likelihood.
+
+        Parameters
+        ----------
+        verbose : bool
+            ``True`` for progress output; ``False`` otherwise.
+            Defaults to ``True``.
+        """
+        self._ep.verbose = verbose
+        super(GLMMExpFam, self).fit(verbose=verbose, factr=factr, pgtol=pgtol)
+        self._ep.verbose = False
+
     def fix(self, var_name):
         GLMM.fix(self, var_name)
         self.set_update_approx()
@@ -169,5 +193,33 @@ class GLMMExpFam(GLMM):
         self.set_update_approx()
 
     def value(self):
+        if self._verbose:
+            b = norm(self.beta)
+            d = self.delta
+            s = self.scale
+            print("b={: 10.6} d={: 10.6} s={: 10.6}".format(b, d, s))
         self._update_approx()
         return self._ep.lml()
+
+    def predictive_mean(self, Xstar, ks, kss):
+        mstar = self.mean_star(Xstar)
+        ks = self.covariance_star(ks)
+        m = self.mean()
+        eta = self._ep.posterior.eta
+        tau = self._ep.posterior.tau
+        mu = eta / tau
+        K = GLMM.covariance(self)
+        return mstar + dot(ks, solve(K, mu - m))
+
+    def predictive_covariance(self, Xstar, ks, kss):
+        kss = self.variance_star(kss)
+        ks = self.covariance_star(ks)
+        tau = self._ep.posterior.tau
+        K = GLMM.covariance(self)
+        KT = sum2diag(K, 1 / tau)
+        ktk = solve(KT, ks.T)
+        b = []
+        for i in range(len(kss)):
+            b += [dot(ks[i, :], ktk[:, i])]
+        b = asarray(b)
+        return kss - b
