@@ -1,16 +1,26 @@
 import pytest
-from numpy import asarray, ascontiguousarray, dot, ones, sqrt, zeros
+from numpy import (
+    arange,
+    asarray,
+    ascontiguousarray,
+    corrcoef,
+    dot,
+    eye,
+    ones,
+    sqrt,
+    zeros,
+)
 from numpy.random import RandomState
-from numpy.testing import assert_allclose
-from numpy_sugar.linalg import economic_qs, economic_qs_linear
-from optimix import check_grad
+from numpy.testing import assert_allclose, assert_
 
 from glimix_core.example import linear_eye_cov, nsamples
 from glimix_core.glmm import GLMMExpFam, GLMMNormal
 from glimix_core.random import bernoulli_sample
+from numpy_sugar.linalg import economic_qs, economic_qs_linear
+from optimix import check_grad
 
-ATOL = 1e-5
-RTOL = 1e-4
+ATOL = 1e-3
+RTOL = 1e-3
 
 
 def test_glmmexpfam_layout():
@@ -20,11 +30,11 @@ def test_glmmexpfam_layout():
     QS = economic_qs(K)
 
     with pytest.raises(ValueError):
-        GLMMExpFam(y, 'poisson', X, QS=QS)
+        GLMMExpFam(y, "poisson", X, QS=QS)
 
     y = asarray([1.0])
     with pytest.raises(ValueError):
-        GLMMExpFam(y, 'poisson', X, QS=QS)
+        GLMMExpFam(y, "poisson", X, QS=QS)
 
 
 def test_glmmexpfam_copy():
@@ -40,7 +50,7 @@ def test_glmmexpfam_copy():
         nsuc[i] += sum(z[i] + 0.2 * random.randn(ni) > 0)
 
     ntri = ascontiguousarray(ntri)
-    glmm0 = GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+    glmm0 = GLMMExpFam((nsuc, ntri), "binomial", X, QS)
 
     assert_allclose(glmm0.lml(), -29.10216812909928, atol=ATOL, rtol=RTOL)
     glmm0.fit(verbose=False)
@@ -72,7 +82,7 @@ def test_glmmexpfam_precise():
     ntri = random.randint(1, 30, nsamples())
     nsuc = [random.randint(0, i) for i in ntri]
 
-    glmm = GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+    glmm = GLMMExpFam((nsuc, ntri), "binomial", X, QS)
     glmm.beta = asarray([1.0, 0, 0.5, 0.1, 0.4])
 
     glmm.scale = 1.0
@@ -105,18 +115,18 @@ def test_glmmexpfam_glmmnormal_get_fast_scanner():
     glmm = GLMMNormal(eta, tau, X, QS)
     glmm.fit(verbose=False)
     want = [-0.08228058, -0.03910674, 0.04226152, -0.05893827, 0.01718722]
-    assert_allclose(glmm.beta, want, atol=1e-6, rtol=1e-6)
-    assert_allclose(0.001, glmm.scale, atol=1e-6, rtol=1e-6)
-    assert_allclose(0.999999994119, glmm.delta, atol=1e-6, rtol=1e-6)
+    assert_allclose(glmm.beta, want, atol=1e-3, rtol=1e-3)
+    assert_allclose(0.001, glmm.scale, atol=1e-3, rtol=1e-3)
+    assert_allclose(0.999999994119, glmm.delta, atol=1e-3, rtol=1e-3)
 
     scanner = glmm.get_fast_scanner()
     scanner.set_scale(1.0)
     lmls, effect_sizes = scanner.fast_scan(X, verbose=False)
     want = [-4.75845, -4.75845, -4.75845, -4.75845, -4.75845]
-    assert_allclose(lmls, want, atol=1e-6, rtol=1e-6)
+    assert_allclose(lmls, want, atol=1e-2, rtol=1e-2)
 
     want = [-0.04114, -0.019553, 0.021131, -0.029469, 0.008593]
-    assert_allclose(effect_sizes, want, atol=1e-6, rtol=1e-6)
+    assert_allclose(effect_sizes, want, atol=1e-2, rtol=1e-2)
 
 
 def test_glmmexpfam_delta0():
@@ -128,7 +138,7 @@ def test_glmmexpfam_delta0():
     ntri = random.randint(1, 30, nsamples())
     nsuc = [random.randint(0, i) for i in ntri]
 
-    glmm = GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+    glmm = GLMMExpFam((nsuc, ntri), "binomial", X, QS)
     glmm.beta = asarray([1.0, 0, 0.5, 0.1, 0.4])
 
     glmm.delta = 0
@@ -146,7 +156,7 @@ def test_glmmexpfam_delta1():
     ntri = random.randint(1, 30, nsamples())
     nsuc = [random.randint(0, i) for i in ntri]
 
-    glmm = GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+    glmm = GLMMExpFam((nsuc, ntri), "binomial", X, QS)
     glmm.beta = asarray([1.0, 0, 0.5, 0.1, 0.4])
 
     glmm.delta = 1
@@ -165,7 +175,7 @@ def test_glmmexpfam_wrong_qs():
     nsuc = [random.randint(0, i) for i in ntri]
 
     with pytest.raises(ValueError):
-        GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+        GLMMExpFam((nsuc, ntri), "binomial", X, QS)
 
 
 def test_glmmexpfam_optimize():
@@ -181,18 +191,18 @@ def test_glmmexpfam_optimize():
         nsuc[i] += sum(z[i] + 0.2 * random.randn(ni) > 0)
 
     ntri = ascontiguousarray(ntri)
-    glmm = GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+    glmm = GLMMExpFam((nsuc, ntri), "binomial", X, QS)
 
     assert_allclose(glmm.lml(), -29.102168129099287, atol=ATOL, rtol=RTOL)
-    glmm.fix('beta')
-    glmm.fix('scale')
+    glmm.fix("beta")
+    glmm.fix("scale")
 
     glmm.fit(verbose=False)
 
     assert_allclose(glmm.lml(), -27.635788105778012, atol=ATOL, rtol=RTOL)
 
-    glmm.unfix('beta')
-    glmm.unfix('scale')
+    glmm.unfix("beta")
+    glmm.unfix("scale")
 
     glmm.fit(verbose=False)
 
@@ -212,7 +222,7 @@ def test_glmmexpfam_optimize_low_rank():
         nsuc[i] += sum(z[i] + 0.2 * random.randn(ni) > 0)
 
     ntri = ascontiguousarray(ntri)
-    glmm = GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+    glmm = GLMMExpFam((nsuc, ntri), "binomial", X, QS)
 
     assert_allclose(glmm.lml(), -18.60476792256323, atol=ATOL, rtol=RTOL)
     glmm.fit(verbose=False)
@@ -224,7 +234,7 @@ def test_glmmexpfam_bernoulli_problematic():
     N = 30
     G = random.randn(N, N + 50)
     y = bernoulli_sample(0.0, G, random_state=random)
-    y = (y, )
+    y = (y,)
 
     G = ascontiguousarray(G, dtype=float)
     _stdnorm(G, 0, out=G)
@@ -235,9 +245,9 @@ def test_glmmexpfam_bernoulli_problematic():
     S0 /= S0.mean()
 
     X = ones((len(y[0]), 1))
-    model = GLMMExpFam(y, 'bernoulli', X, QS=(QS[0], QS[1]))
+    model = GLMMExpFam(y, "bernoulli", X, QS=(QS[0], QS[1]))
     model.delta = 0
-    model.fix('delta')
+    model.fix("delta")
     model.fit(verbose=False)
     assert_allclose(model.lml(), -20.727007958026853, atol=ATOL, rtol=RTOL)
     assert_allclose(model.delta, 0, atol=1e-3)
@@ -245,6 +255,72 @@ def test_glmmexpfam_bernoulli_problematic():
     assert_allclose(model.beta, [-0.00247856564728], atol=ATOL, rtol=RTOL)
 
 
+def test_glmmexpfam_bernoulli_probit_problematic():
+    random = RandomState(1)
+    N = 30
+    G = random.randn(N, N + 50)
+    y = bernoulli_sample(0.0, G, random_state=random)
+    y = (y,)
+
+    G = ascontiguousarray(G, dtype=float)
+    _stdnorm(G, 0, out=G)
+    G /= sqrt(G.shape[1])
+
+    QS = economic_qs_linear(G)
+    S0 = QS[1]
+    S0 /= S0.mean()
+
+    X = ones((len(y[0]), 1))
+    model = GLMMExpFam(y, "probit", X, QS=(QS[0], QS[1]))
+    model.delta = 0
+    model.fix("delta")
+    model.fit(verbose=False)
+    assert_allclose(model.lml(), -20.725623168378615, atol=ATOL, rtol=RTOL)
+    assert_allclose(model.delta, 0.0001220703125, atol=1e-3)
+    assert_allclose(model.scale, 0.33022865011938707, atol=ATOL, rtol=RTOL)
+    assert_allclose(model.beta, [-0.002617161564786044], atol=ATOL, rtol=RTOL)
+
+    h20 = model.scale * (1 - model.delta) / (model.scale + 1)
+
+    model.unfix("delta")
+    model.delta = 0.5
+    model.scale = 1.0
+    model.fit(verbose=False)
+
+    assert_allclose(model.lml(), -20.725623168378522, atol=ATOL, rtol=RTOL)
+    assert_allclose(model.delta, 0.5017852859580029, atol=1e-3)
+    assert_allclose(model.scale, 0.9928931515372, atol=ATOL, rtol=RTOL)
+    assert_allclose(model.beta, [-0.003203427206253548], atol=ATOL, rtol=RTOL)
+
+    h21 = model.scale * (1 - model.delta) / (model.scale + 1)
+
+    assert_allclose(h20, h21, atol=ATOL, rtol=RTOL)
+
+
+def test_glmmexpfam_bernoulli_probit_assure_delta_fixed():
+    random = RandomState(1)
+    N = 10
+    G = random.randn(N, N + 50)
+    y = bernoulli_sample(0.0, G, random_state=random)
+    y = (y,)
+
+    G = ascontiguousarray(G, dtype=float)
+    _stdnorm(G, 0, out=G)
+    G /= sqrt(G.shape[1])
+
+    QS = economic_qs_linear(G)
+    S0 = QS[1]
+    S0 /= S0.mean()
+
+    X = ones((len(y[0]), 1))
+    model = GLMMExpFam(y, "probit", X, QS=(QS[0], QS[1]))
+    model.fit(verbose=False)
+
+    assert_allclose(model.lml(), -6.108751595773174, rtol=RTOL)
+    assert_allclose(model.delta, 1.4901161193847673e-08, atol=1e-5)
+    assert_(model.isfixed("logitdelta"))
+
+
 def _stdnorm(X, axis=None, out=None):
     X = ascontiguousarray(X)
     if out is None:
@@ -283,7 +359,7 @@ def test_glmmexpfam_binomial_pheno_list():
     y = [successes, ntrials]
 
     QS = economic_qs(K)
-    glmm = GLMMExpFam(y, 'binomial', X, QS)
+    glmm = GLMMExpFam(y, "binomial", X, QS)
     glmm.fit(verbose=False)
 
     assert_allclose(glmm.lml(), -11.43920790567486)
@@ -298,7 +374,7 @@ def test_glmmexpfam_scale_very_low():
     ntri = random.randint(1, 30, nsamples())
     nsuc = [random.randint(0, i) for i in ntri]
 
-    glmm = GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+    glmm = GLMMExpFam((nsuc, ntri), "binomial", X, QS)
     glmm.beta = asarray([1.0, 0, 0.5, 0.1, 0.4])
 
     glmm.scale = 1e-3
@@ -316,7 +392,7 @@ def test_glmmexpfam_scale_very_high():
     ntri = random.randint(1, 30, nsamples())
     nsuc = [random.randint(0, i) for i in ntri]
 
-    glmm = GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+    glmm = GLMMExpFam((nsuc, ntri), "binomial", X, QS)
     glmm.beta = asarray([1.0, 0, 0.5, 0.1, 0.4])
 
     glmm.scale = 30.
@@ -336,21 +412,106 @@ def test_glmmexpfam_delta_one_zero():
     ntri = random.randint(1, 30, n)
     nsuc = [random.randint(0, i) for i in ntri]
 
-    glmm = GLMMExpFam((nsuc, ntri), 'binomial', X, QS)
+    glmm = GLMMExpFam((nsuc, ntri), "binomial", X, QS)
     glmm.beta = asarray([1.0, 0, 0.5, 0.1, 0.4, -0.2])
 
     glmm.delta = 0
-    assert_allclose(glmm.lml(), -113.22468572634763)
+    assert_allclose(glmm.lml(), -113.24570457063275)
     assert_allclose(check_grad(glmm, step=1e-4), 0, atol=1e-2)
 
     glmm.fit(verbose=False)
-    assert_allclose(glmm.lml(), -98.21142075640205, atol=ATOL, rtol=RTOL)
-    assert_allclose(glmm.delta, 0.00012363396798507502, atol=ATOL, rtol=RTOL)
+    assert_allclose(glmm.lml(), -98.21144899310399, atol=ATOL, rtol=RTOL)
+    assert_allclose(glmm.delta, 0, atol=ATOL, rtol=RTOL)
 
     glmm.delta = 1
-    assert_allclose(glmm.lml(), -98.00060746015443, atol=ATOL, rtol=RTOL)
+    assert_allclose(glmm.lml(), -98.00058169240869, atol=ATOL, rtol=RTOL)
     assert_allclose(check_grad(glmm, step=1e-4), 0, atol=1e-1)
 
     glmm.fit(verbose=False)
-    assert_allclose(glmm.lml(), -72.82680932955623, atol=ATOL, rtol=RTOL)
-    assert_allclose(glmm.delta, 0.9999999998430158, atol=ATOL, rtol=RTOL)
+
+    assert_allclose(glmm.lml(), -72.82680948264196, atol=ATOL, rtol=RTOL)
+    assert_allclose(glmm.delta, 0.9999999850988439, atol=ATOL, rtol=RTOL)
+
+
+def test_glmmexpfam_predict():
+
+    random = RandomState(4)
+    n = 100
+    p = n + 1
+
+    X = ones((n, 2))
+    X[:, 1] = random.randn(n)
+
+    G = random.randn(n, p)
+    G /= G.std(0)
+    G -= G.mean(0)
+    G /= sqrt(p)
+    K = dot(G, G.T)
+
+    i = asarray(arange(0, n), int)
+    si = random.choice(i, n, replace=False)
+    ntest = int(n // 5)
+    itrain = si[:-ntest]
+    itest = si[-ntest:]
+
+    Xtrain = X[itrain, :]
+    Ktrain = K[itrain, :][:, itrain]
+
+    Xtest = X[itest, :]
+
+    beta = random.randn(2)
+    z = random.multivariate_normal(dot(X, beta), 0.9 * K + 0.1 * eye(n))
+
+    ntri = random.randint(1, 100, n)
+    nsuc = zeros(n, dtype=int)
+    for (i, ni) in enumerate(ntri):
+        nsuc[i] += sum(z[i] + 0.2 * random.randn(ni) > 0)
+
+    ntri = ascontiguousarray(ntri)
+
+    QStrain = economic_qs(Ktrain)
+    nsuc_train = ascontiguousarray(nsuc[itrain])
+    ntri_train = ascontiguousarray(ntri[itrain])
+
+    nsuc_test = ascontiguousarray(nsuc[itest])
+    ntri_test = ascontiguousarray(ntri[itest])
+
+    glmm = GLMMExpFam((nsuc_train, ntri_train), "binomial", Xtrain, QStrain)
+    glmm.fit(verbose=False)
+    ks = K[itest, :][:, itrain]
+    kss = asarray([K[i, i] for i in itest])
+    pm = glmm.predictive_mean(Xtest, ks, kss)
+    pk = glmm.predictive_covariance(Xtest, ks, kss)
+    r = nsuc_test / ntri_test
+    assert_(corrcoef([pm, r])[0, 1] > 0.8)
+    assert_allclose(pk[0], 54.263491276875726)
+
+
+def test_glmmexpfam_qs_none():
+    random = RandomState(0)
+    X = random.randn(nsamples(), 5)
+    K = linear_eye_cov().feed().value()
+    z = random.multivariate_normal(0.2 * ones(nsamples()), K)
+
+    ntri = random.randint(1, 30, nsamples())
+    nsuc = zeros(nsamples(), dtype=int)
+    for (i, ni) in enumerate(ntri):
+        nsuc[i] += sum(z[i] + 0.2 * random.randn(ni) > 0)
+
+    ntri = ascontiguousarray(ntri)
+    glmm = GLMMExpFam((nsuc, ntri), "binomial", X, None)
+
+    assert_allclose(glmm.lml(), -38.30173374439622, atol=ATOL, rtol=RTOL)
+    glmm.fix("beta")
+    glmm.fix("scale")
+
+    glmm.fit(verbose=False)
+
+    assert_allclose(glmm.lml(), -32.03927471370041, atol=ATOL, rtol=RTOL)
+
+    glmm.unfix("beta")
+    glmm.unfix("scale")
+
+    glmm.fit(verbose=False)
+
+    assert_allclose(glmm.lml(), -19.575736561760586, atol=ATOL, rtol=RTOL)
