@@ -1,5 +1,4 @@
-r"""
-"""
+r"""Linear mixed model package."""
 
 from .lmm import LMM
 from .scan import FastScanner
