@@ -1,14 +1,15 @@
 from __future__ import division
 
-from numpy import clip, errstate, exp, log
-from numpy_sugar import epsilon, is_all_finite
-from optimix import Function, Scalar, maximize_scalar
+from numpy import asarray, clip, dot, exp
+from numpy.linalg import solve
 
-from .core import LMMCore
+from numpy_sugar import epsilon, is_all_finite
+
+from ._core import LMMCore
 from .scan import FastScanner
 
 
-class LMM(LMMCore, Function):
+class LMM(LMMCore):
     r"""Fast Linear Mixed Models inference via maximum likelihood.
 
     It perform inference on the model :eq:`lmm1`, explained in the
@@ -26,7 +27,6 @@ class LMM(LMMCore, Function):
 
     Examples
     --------
-
     .. doctest::
 
         >>> from numpy import array
@@ -68,24 +68,29 @@ class LMM(LMMCore, Function):
         -3.713
     """
 
-    def __init__(self, y, X, QS):
+    def __init__(self, y, X, QS=None):
         LMMCore.__init__(self, y, X, QS)
-        Function.__init__(self, logistic=Scalar(0.0))
 
         if not is_all_finite(y):
-            raise ValueError("There are non-finite values in the phenotype.")
+            raise ValueError("There are non-finite values in the outcome.")
 
         self.set_nodata()
 
     def _get_delta(self):
-        v = clip(self.variables().get('logistic').value, -20, 20)
+        v = clip(self.variables().get("logistic").value, -20, 20)
         x = 1 / (1 + exp(-v))
-        return clip(x, 1e-5, 1 - 1e-5)
+        return clip(x, epsilon.tiny, 1 - epsilon.tiny)
 
     @property
     def beta(self):
+        r"""Fixed-effect sizes."""
         return LMMCore.beta.fget(self)
 
+    @beta.setter
+    def beta(self, beta):
+        r"""Set fixed-effect sizes."""
+        LMMCore.beta.fset(self, beta)
+
     def copy(self):
         r"""Return a copy of this object.
 
@@ -101,30 +106,18 @@ class LMM(LMMCore, Function):
         o = LMM.__new__(LMM)
 
         LMMCore.__init__(o, self._y, self.X, self._QS)
-        Function.__init__(
-            o, logistic=Scalar(self.variables().get('logistic').value))
+        o.delta = self.delta
+        if self.isfixed("delta"):
+            o.fix("delta")
 
-        setattr(o, '_fix_scale', self._fix_scale)
-        setattr(o, '_scale', self._scale)
+        setattr(o, "_fix_scale", self._fix_scale)
+        setattr(o, "_scale", self._scale)
+        setattr(o, "_fix_beta", self._fix_beta)
+        setattr(o, "_verbose", self._verbose)
 
         o.set_nodata()
         return o
 
-    @property
-    def delta(self):
-        r"""Variance ratio between ``K`` and ``I``."""
-        # return self._delta
-        v = float(self.variables().get('logistic').value)
-        with errstate(over='ignore', under='ignore'):
-            v = 1 / (1 + exp(-v))
-        return clip(v, epsilon.tiny, 1 - epsilon.tiny)
-
-    @delta.setter
-    def delta(self, delta):
-        delta = clip(delta, epsilon.tiny, 1 - epsilon.tiny)
-        # self._delta = delta
-        self.variables().set(dict(logistic=log(delta / (1 - delta))))
-
     def isfixed(self, var_name):
         r"""Return whether a variable it is fixed or not.
 
@@ -138,10 +131,13 @@ class LMM(LMMCore, Function):
         bool
             ``True`` if fixed; ``False`` otherwise.
         """
-        if var_name not in ['delta', 'scale']:
-            raise ValueError("Possible values are 'delta' and 'scale'.")
-        if var_name == 'delta':
-            return Function.isfixed(self, 'logistic')
+        if var_name not in ["delta", "scale", "beta"]:
+            msg = "Possible values are 'delta', 'scale', and 'beta'."
+            raise ValueError(msg)
+        if var_name == "delta":
+            return super(LMM, self).isfixed("logistic")
+        if var_name == "beta":
+            return self._fix_beta
         return self._fix_scale
 
     @property
@@ -175,9 +171,12 @@ class LMM(LMMCore, Function):
             ``True`` for progress output; ``False`` otherwise.
             Defaults to ``True``.
         """
-        maximize_scalar(self, 'LMM', verbose=verbose)
+        self._verbose = verbose
+        if not self.isfixed("delta"):
+            self.feed().maximize_scalar(verbose=verbose)
         self._update()
         self.delta = self._get_delta()
+        self._verbose = False
 
     def fix(self, var_name):
         r"""Disable the optimisation of a given variable.
@@ -187,11 +186,14 @@ class LMM(LMMCore, Function):
         var_name : str
             Possible values are `delta` and `scale`.
         """
-        if var_name not in ['delta', 'scale']:
-            raise ValueError("Possible values are 'delta' and 'scale'.")
+        if var_name not in ["delta", "scale", "beta"]:
+            msg = "Possible values are 'delta', 'scale', and 'beta'."
+            raise ValueError(msg)
 
-        if var_name == 'delta':
-            super(LMM, self).fix('logistic')
+        if var_name == "delta":
+            super(LMM, self).fix("logistic")
+        elif var_name == "beta":
+            self._fix_beta = True
         else:
             if not self._fix_scale:
                 self._scale = self.scale
@@ -256,10 +258,13 @@ class LMM(LMMCore, Function):
         var_name : str
             Possible values are `delta` and `scale`.
         """
-        if var_name not in ['delta', 'scale']:
-            raise ValueError("Possible values are 'delta' and 'scale'.")
-        if var_name == 'delta':
-            super(LMM, self).unfix('logistic')
+        if var_name not in ["delta", "scale", "beta"]:
+            msg = "Possible values are 'delta', 'scale', and 'beta'."
+            raise ValueError(msg)
+        if var_name == "delta":
+            super(LMM, self).unfix("logistic")
+        elif var_name == "beta":
+            self._fix_beta = False
         else:
             self._fix_scale = False
 
@@ -277,3 +282,21 @@ class LMM(LMMCore, Function):
 
     def gradient(self, *_):
         raise NotImplementedError
+
+    def predictive_mean(self, Xstar, ks, kss):
+        mstar = self.mean_star(Xstar)
+        ks = self.covariance_star(ks)
+        m = self.mean
+        K = LMMCore.covariance(self)
+        return mstar + dot(ks, solve(K, self._y - m))
+
+    def predictive_covariance(self, Xstar, ks, kss):
+        kss = self.variance_star(kss)
+        ks = self.covariance_star(ks)
+        K = LMMCore.covariance(self)
+        ktk = solve(K, ks.T)
+        b = []
+        for i in range(len(kss)):
+            b += [dot(ks[i, :], ktk[:, i])]
+        b = asarray(b)
+        return kss - b
