@@ -1,14 +1,13 @@
 from __future__ import division
 
 import pytest
-from numpy import arange, inf, nan, ones, sqrt, zeros
+from numpy import arange, corrcoef, dot, inf, nan, ones, sqrt, zeros
 from numpy.random import RandomState
 from numpy.testing import assert_, assert_allclose
 
 from glimix_core.cov import EyeCov, LinearCov, SumCov
 from glimix_core.lik import DeltaProdLik
 from glimix_core.lmm import LMM
-from glimix_core.lmm.core import LMMCore
 from glimix_core.mean import OffsetMean
 from glimix_core.random import GGPSampler
 from numpy_sugar.linalg import economic_qs_linear
@@ -69,6 +68,26 @@ def test_lmm_fix_unfix():
     with pytest.raises(ValueError):
         lmm.isfixed('deltaa')
 
+    lmm = LMM(y, ones((n, 1)), QS)
+    lmm.fix('beta')
+    lmm.beta = [1.5]
+    assert_allclose(lmm.lml(), -59.02992868385325)
+    assert_allclose(lmm.beta[0], 1.5)
+    lmm.fit(verbose=False)
+    assert_allclose(lmm.lml(), -56.56425503034913)
+    assert_allclose(lmm.beta[0], 1.5)
+    lmm.unfix('beta')
+    assert_allclose(lmm.lml(), -52.2963882611983)
+    assert_allclose(lmm.beta[0], 0.7065598068496929)
+    lmm.fit(verbose=False)
+    assert_allclose(lmm.lml(), -51.84396136865775)
+    assert_allclose(lmm.beta[0], 0.7065598068496929)
+    lmm.fix('beta')
+    lmm.beta = lmm.beta[0] + 0.01
+    assert_allclose(lmm.lml(), -51.84505787833421)
+    lmm.beta = lmm.beta[0] - 2 * 0.01
+    assert_allclose(lmm.lml(), -51.84505787833422)
+
 
 def test_lmm_unique_outcome():
     random = RandomState(9458)
@@ -108,20 +127,6 @@ def test_lmm_nonfinite_outcome():
         LMM(y, ones((N, 1)), QS)
 
 
-def test_lmm_lmmcore_interface():
-    random = RandomState(9458)
-    N = 5
-    QS = economic_qs_linear(random.randn(N, N + 1))
-    y = zeros(N)
-
-    lmmc = LMMCore(y, ones((N, 1)), QS)
-    with pytest.raises(NotImplementedError):
-        print(lmmc.delta)
-
-    with pytest.raises(NotImplementedError):
-        lmmc.delta = 1
-
-
 def test_lmm_redundant_covariates_fullrank():
     random = RandomState(9458)
     n = 30
@@ -203,3 +208,63 @@ def _covariates_sample(random, n, p):
     X /= X.std(0)
     X /= sqrt(X.shape[1])
     return X
+
+
+def test_lmm_prediction():
+    random = RandomState(9458)
+    n = 30
+    X = _covariates_sample(random, n, n + 1)
+
+    offset = 1.0
+
+    y = _outcome_sample(random, offset, X)
+
+    QS = economic_qs_linear(X)
+
+    lmm = LMM(y, ones((n, 1)), QS)
+
+    lmm.fit(verbose=False)
+
+    K = dot(X, X.T)
+    pm = lmm.predictive_mean(ones((n, 1)), K, K.diagonal())
+    assert_allclose(corrcoef(y, pm)[0, 1], 0.8358820971891354)
+
+
+def test_lmm_iid_prior():
+    random = RandomState(9458)
+    n = 30
+    X = _covariates_sample(random, n, n + 1)
+
+    offset = 1.0
+
+    y = _outcome_sample(random, offset, X)
+
+    lmm = LMM(y, ones((n, 1)), None)
+
+    assert_(lmm.isfixed('delta'))
+
+    assert_allclose(lmm.lml(), -52.29638826846387)
+    lmm.fit(verbose=False)
+
+    assert_allclose(lmm.beta[0], 0.7065598068496929)
+    assert_allclose(lmm.scale, 1.9127630509027338)
+    assert_allclose(lmm.delta, 0.9999999999999998)
+    assert_allclose(lmm.lml(), -52.29638826846388)
+
+
+def test_lmm_zero_rank_covariates():
+    random = RandomState(9458)
+    n = 30
+    X = _covariates_sample(random, n, n + 1)
+
+    offset = 1.0
+
+    y = _outcome_sample(random, offset, X)
+
+    QS = economic_qs_linear(X)
+    lmm = LMM(y, zeros((n, 1)), QS)
+    lmm.fit(verbose=False)
+    assert_allclose(lmm.lml(), -55.774936794143315)
+    assert_allclose(lmm.scale, 2.411989807718206)
+    assert_allclose(lmm.delta, 0.9999999979388463)
+    assert_allclose(lmm.beta, [0])
